/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.system.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.excel.UserExcel;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

/**
 * Mapper 接口
 *
 * @author Chill
 */
public interface UserMapper extends BaseMapper<User> {

	/**
	 * 自定义分页
	 *
	 * @param page
	 * @param user
	 * @param deptIdList
	 * @param tenantId
	 * @return
	 */
	List<User> selectUserPage(IPage<User> page, @Param("user") User user, @Param("deptIdList") List<Long> deptIdList, @Param("tenantId") String tenantId);

	/**
	 * 获取用户
	 *
	 * @param tenantId
	 * @param account
	 * @param password
	 * @return
	 */
	@Deprecated
	User getUser(String tenantId, String account, String password);
	
	User login(String account, String password);
	
	/**
	 * 获取导出用户数据
	 *
	 * @param queryWrapper
	 * @return
	 */
	List<UserExcel> exportUser(@Param("ew") Wrapper<User> queryWrapper);
	
	@Select("select * from blade_user where is_deleted = 0 and find_in_set('${value}', dept_id)")
	List<User> selectUserIdListByDeptId(Long deptId);
	
	@Select("select id from blade_user where is_deleted = 0 and find_in_set('${value}', role_id)")
	List<User> selectUserIdListByRoleId(Long roleId);
	
	@Select({
		"<script>",
		"select * from blade_user where is_deleted = 0 and id in (",
		"select user_id from blade_user_dept where dept_id in",
		"<foreach collection='deptIdList' item='item' open='(' separator=',' close=')'>",
		"#{item}",
		"</foreach>",
		")",
		"</script>"
	})
	List<User> selectUserIdListByDeptIdList(@Param("deptIdList")List<Long> deptIdList);

}
