/*
 * Copyright (c) 2018-2028, AoKunSang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.modules.system.service.normal;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.common.cache.ParamCache;
import org.springblade.core.constant.BladeConstant;
import org.springblade.core.constant.RoleConstant;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.auth.enums.UserEnum;
import org.springblade.modules.system.entity.DataPermission;
import org.springblade.modules.system.entity.DataPermissionUser;
import org.springblade.modules.system.entity.Dept;
import org.springblade.modules.system.entity.PermEntity;
import org.springblade.modules.system.entity.PermJsonEntity;
import org.springblade.modules.system.entity.Post;
import org.springblade.modules.system.entity.Role;
import org.springblade.modules.system.entity.ThirdLoginConfig;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.VisualSubscribeNotifyConf;
import org.springblade.modules.system.service.IDeptService;
import org.springblade.modules.system.service.IPostService;
import org.springblade.modules.system.service.IRoleService;
import org.springblade.modules.system.service.IThirdLoginConfigService;
import org.springblade.modules.system.service.IUserService;
import org.springblade.modules.system.service.VisualSubscribeNotifyConfService;
import org.springblade.system.client.dto.TenantInitParams;
import org.springblade.system.client.dto.TenantInitReturn;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import static org.springblade.common.constant.TenantConstant.DEFAULT_PASSWORD;
import static org.springblade.common.constant.TenantConstant.PASSWORD_KEY;

/**
 * 租户初始化
 *
 * @author wookvn
 * @datetime 2025-01-08 14:55:31
 */
@Component
public class SysTenantInitService {

    @Autowired
    private IRoleService roleService;
    @Autowired
    private IDeptService deptService;
    @Autowired
    private IPostService postService;
    @Autowired
    private IUserService userService;
    @Autowired
    private DataPermissionService dataPermissionService;
    @Autowired
    private DataPermissionUserService dataPermissionUserService;
    @Autowired
    private VisualSubscribeNotifyConfService visualSubscribeNotifyConfService;
    @Autowired
    private IThirdLoginConfigService thirdLoginConfigService;

    /**
     * <p>租户初始化</p>
     * @author wookvn
     * @datetime 2025-01-08 14:56:20
     * @param params: 初始化参数
     * @return org.springblade.system.client.dto.TenantInitReturn
     **/
    @Transactional(rollbackFor = Exception.class)
    public TenantInitReturn tenantInit(TenantInitParams params) {
        TenantInitReturn tenantInitReturn = new TenantInitReturn();
        String tenantId = params.getTenantId();
        // 新建租户对应的默认角色
        Role role = new Role();
        role.setTenantId(tenantId);
        role.setParentId(BladeConstant.TOP_PARENT_ID);
        role.setRoleName("管理员");
        role.setRoleAlias(RoleConstant.ADMIN);
        role.setSort(2);
        role.setIsDeleted(BladeConstant.DB_NOT_DELETED);
        roleService.save(role);
        // 新建租户对应的默认部门
        Dept dept = new Dept();
        dept.setTenantId(tenantId);
        dept.setParentId(BladeConstant.TOP_PARENT_ID);
        dept.setAncestors(String.valueOf(BladeConstant.TOP_PARENT_ID));
        dept.setDeptName(params.getTenantName());
        dept.setFullName(params.getTenantName());
        dept.setDeptCategory(1);
        dept.setSort(2);
        dept.setIsDeleted(BladeConstant.DB_NOT_DELETED);
        deptService.save(dept);
        // 新建租户对应的默认岗位
        Post post = new Post();
        post.setTenantId(tenantId);
        post.setCategory(1);
        post.setPostCode("ceo");
        post.setPostName("首席执行官");
        post.setSort(1);
        postService.save(post);
        //添加超级权限组
        DataPermission dataPermission = new DataPermission();
        dataPermission.setCreateTime(new Date());
        dataPermission.setTenantId(tenantId);
        dataPermission.setSuperStatus(1);
        dataPermission.setTypeId(0);
        List<PermEntity> list = new LinkedList<>();
        List<PermJsonEntity> jsonList = new LinkedList<>();
        PermJsonEntity json = new PermJsonEntity();
        jsonList.add(json);
        PermEntity permEntity = new PermEntity();
        permEntity.setDpList(jsonList);
        permEntity.setUserList(jsonList);
        permEntity.setRoleList(jsonList);
        list.add(permEntity);
        dataPermission.setPermJson(JSONUtil.parseArray(list).toString());
        dataPermission.setCreateUserId(0L);
        dataPermission.setPermName("超级权限组");
        dataPermissionService.save(dataPermission);
        // 新建租户对应的默认管理用户
        User user = new User();
        user.setTenantId(tenantId);
        user.setName("admin");
        user.setRealName("管理员");
        user.setAccount(params.getContactNumber());
        user.setPhone(params.getContactNumber());
        // 获取参数配置的密码
        String password = Func.toStr(ParamCache.getValue(PASSWORD_KEY), DEFAULT_PASSWORD);
        user.setPassword(password);
        user.setRoleId(String.valueOf(role.getId()));
        user.setDeptId(String.valueOf(dept.getId()));
        user.setPostId(String.valueOf(post.getId()));
        user.setBirthday(new Date());
        user.setSex(1);
        user.setUserType(UserEnum.WEB.getCategory());
        user.setIsDeleted(BladeConstant.DB_NOT_DELETED);
        user.setAuditLevel(1);
        user.setIsLogin(0);
        user.setIsWxLogin(0);
        userService.submit(user);
        tenantInitReturn.setAdminId(user.getId());
        //把该用户添加到超级权限组中
        DataPermissionUser dataPermissionUser = new DataPermissionUser();
        dataPermissionUser.setUserId(user.getId());
        dataPermissionUser.setDataPermId(dataPermission.getId());
        dataPermissionUser.setUserStatus(1);
        dataPermissionUserService.save(dataPermissionUser);
        // 初始化开放平台配置
        List<VisualSubscribeNotifyConf> notifyConfs = visualSubscribeNotifyConfService.list(Wrappers.<VisualSubscribeNotifyConf>lambdaQuery()
                .eq(VisualSubscribeNotifyConf::getTenantId, BladeConstant.ADMIN_TENANT_ID));
        notifyConfs.forEach(conf -> {
            conf.setId(null);
            conf.setTenantId(tenantId);
        });
        visualSubscribeNotifyConfService.saveBatch(notifyConfs);
        // 初始化三方登录配置
        List<ThirdLoginConfig> loginConfigs = thirdLoginConfigService.list(Wrappers.<ThirdLoginConfig>lambdaQuery()
                .eq(ThirdLoginConfig::getTenantId, BladeConstant.ADMIN_TENANT_ID));
        loginConfigs.forEach(login -> {
            login.setId(null);
            login.setTenantId(tenantId);
            login.setStatus(1);
        });
        thirdLoginConfigService.saveBatch(loginConfigs);
        return tenantInitReturn;
    }
}
